<?php
/**
 * Default copyright laws apply.
 *
 * Veebipoed retains all rights to this source code and nobody else may reproduce, distribute
 * or create derivative works from this work.
 *
 * Permitted:
 * - Commercial Use
 * This software and derivatives may be used for commercial purposes.
 * - Private Use
 * You may use and modify the software without distributing it.
 *
 * Forbidden:
 * - Distribution
 * You may not distribute this software.
 * - Modification
 * This software may not be modified.
 * - Sublicensing
 * You may not grant a sublicense to modify and distribute this software to third parties not included in the license.
 *
 *  @author    Veebipoed.ee
 *  @copyright 2020 Veebipoed.ee
 *  @license   Veebipoed.ee
 */

class LHVLeaseContract extends ObjectModel
{
    public $id_cart;
    public $idcode;
    public $code_issuer;
    public $full_name;
    public $contract_status;
    public $contract_number;
    public $down_payment;
    public $contract_fee;
    public $email;
    public $phone;
    public $country_code;
    public $city;
    public $street_address;
    public $postal_code;
    public $signing_date;

    public static $definition = array(
        'table' => 'lhvlease',
        'primary' => 'id_lhvlease',
        'multilang' => false,
        'fields' => array(
            'id_cart' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'required' => true),
            'idcode' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true, 'size' => 32),
            'code_issuer' => array(
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
                'required' => true,
                'size' => 32,
            ),
            'full_name' => array(
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
                'required' => true,
                'size' => 255,
            ),
            'contract_status' => array(
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
                'required' => true,
                'size' => 255,
            ),
            'contract_number' => array(
                'type' => self::TYPE_INT,
                'validate' => 'isInt',
                'required' => true,
                'size' => 255,
            ),
            'down_payment' => array('type' => self::TYPE_FLOAT, 'validate' => 'isPrice', 'required' => true),
            'contract_fee' => array('type' => self::TYPE_FLOAT, 'validate' => 'isPrice', 'required' => true),
            'email' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true, 'size' => 255),
            'phone' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true, 'size' => 255),
            'country_code' => array(
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
                'required' => true,
                'size' => 255,
            ),
            'city' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true, 'size' => 255),
            'street_address' => array(
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
                'required' => true,
                'size' => 255,
            ),
            'postal_code' => array(
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
                'required' => true,
                'size' => 255,
            ),
            'signing_date' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'size' => 255),
        ),
    );

    public static function getContractByCardId($id_cart)
    {
        $query = new DbQuery();
        $query->from('lhvlease');
        $query->where('`id_cart` = '.(int)$id_cart);
        $result = Db::getInstance()->getRow($query);

        $object = new self();

        if (!empty($result)) {
            foreach ($result as $key => $value) {
                $object->{$key} = $value;
            }
            $object->id = $result['id_lhvlease'];
        }

        return $object;
    }
}
