<?php
/**
 * Default copyright laws apply.
 *
 * Veebipoed retains all rights to this source code and nobody else may reproduce, distribute
 * or create derivative works from this work.
 *
 * Permitted:
 * - Commercial Use
 * This software and derivatives may be used for commercial purposes.
 * - Private Use
 * You may use and modify the software without distributing it.
 *
 * Forbidden:
 * - Distribution
 * You may not distribute this software.
 * - Modification
 * This software may not be modified.
 * - Sublicensing
 * You may not grant a sublicense to modify and distribute this software to third parties not included in the license.
 *
 *  @author    Veebipoed.ee
 *  @copyright 2020 Veebipoed.ee
 *  @license   Veebipoed.ee
 */

class lhvleasevalModuleFrontController extends ModuleFrontController
{
    public $ssl = true;

    const VERIFIED = 1;

    const MAIL_CUSTOMER = 1;
    const MAIL_ADMIN = 2;

    const CONTRACT_SUCCESS = 5111;
    const CONTRACT_PENDING = 5112;
    const CONTRACT_ERROR = 5113;

    private $contract_state;
    private $order = null;

    public function initContent()
    {
        parent::initContent();

        $verification = (int)$this->vertifySign('VK_MAC');
        $cart = new Cart((int)Tools::getValue('VK_STAMP', 0));
        $this->contract_state = (int)Tools::getValue('VK_SERVICE', 0);
        $contract_data = Tools::getValue('VK_DATA');

        if ($verification === self::VERIFIED &&
            in_array($this->contract_state, array(self::CONTRACT_PENDING, self::CONTRACT_SUCCESS)) &&
            Validate::isLoadedObject($cart)
        ) {
            if (!$cart->orderExists()) {
                $xml = simplexml_load_string($contract_data);

                $contract = new LhvLeaseContract();
                $contract->id_cart = (int)$cart->id;
                $contract->idcode = (string)$xml->IdCode;
                $contract->code_issuer = (string)$xml->IdCodeIssuer;
                $contract->full_name = (string)$xml->FullName;
                $contract->contract_status = (string)$xml->ContractStatusCode;
                $contract->contract_number = (int)$xml->ContractNumber;
                $contract->down_payment = (float)$xml->ContractDownPaymentAmount;
                $contract->contract_fee = (float)$xml->ContractFeeAmount;
                $contract->email = (string)$xml->CustomerEmail;
                $contract->phone = (string)$xml->CustomerPhone;
                $contract->country_code = (string)$xml->CustomerCountryCode;
                $contract->city = (string)$xml->CustomerCity;
                $contract->street_address = (string)$xml->CustomerStreetAddress;
                $contract->postal_code = (string)$xml->CustomerPostalCode;
                $contract->signing_date = (string)$xml->SigningDtime;
                $contract->add();

                if ($this->contract_state == self::CONTRACT_PENDING) {
                    $order_state_key = LHVLease::ORDER_STATE_WAITING;
                } else {
                    $order_state_key = LHVLease::ORDER_STATE_SUCCESS;
                }

                $id_order_state = Configuration::get($order_state_key);
             //   $order_state = new OrderState($id_order_state);

                $this->module->validateOrder(
                    $cart->id,
                    $id_order_state,
                    $cart->getOrderTotal(),
                    $this->module->displayName,
                    null,
                    array(),
                    null,
                    false,
                    $cart->secure_key
                );

                $order = $this->getOrder($cart->id);
                $this->sendMails($order, self::MAIL_ADMIN);
                $this->sendMails($order, self::MAIL_CUSTOMER);
            }
        } else {
            Tools::redirectLink(__PS_BASE_URI__);
        }

        $order = $this->getOrder($cart->id);

        if (Validate::isLoadedObject($order)) {
            if ((bool)$this->module->getConfig('conf_page')) {
                Tools::redirect($this->getOrderConfPage($order));
            } else {
                if ($this->module->hasOrderState($order, LHVLease::ORDER_STATE_SUCCESS)) {
                    $this->setTemplate('module:lhvlease/views/templates/front/success.tpl');
                } else {
                    if ($this->module->hasOrderState($order, LHVLease::ORDER_STATE_WAITING)) {
                        $this->setTemplate('module:lhvlease/views/templates/front/pending.tpl');
                    } else {
                        $this->setTemplate('module:lhvlease/views/templates/front/nothing.tpl');
                    }
                }
            }
        } else {
            //Valid Request, but something is wrong with the order
            //TODO: Error managment
            Tools::redirectLink(__PS_BASE_URI__);
        }
    }

    private function vertifySign($key)
    {
        $server = $this->module->getConfig('server');

        if (Tools::getIsset($key)) {
            $signature = base64_decode(Tools::getValue($key));
        } else {
            return 2;
        }
        $data = $this->getResponseFields();
        $cert = $this->module->getConfigFromFile($server, 'public_key');

        $publicKey = openssl_get_publickey($cert);
        $out = openssl_verify($data, $signature, $publicKey);
        openssl_free_key($publicKey);

        return $out;
    }

    private function getOrder($id_cart)
    {
        if (is_null($this->order)) {
            $id_order = Order::getOrderByCartId((int)$id_cart);
            $order = new Order($id_order);
            if (Validate::isLoadedObject($order)) {
                $this->order = $order;
            }
        }

        return $this->order;
    }

    private function getResponseFields()
    {
        $array = array();

        if (Tools::getIsset('VK_SERVICE')) {
            $array = array(
                'VK_SERVICE' => Tools::getValue('VK_SERVICE'),
                'VK_VERSION' => Tools::getValue('VK_VERSION'),
                'VK_SND_ID' => Tools::getValue('VK_SND_ID'),
                'VK_REC_ID' => Tools::getValue('VK_REC_ID'),
                'VK_STAMP' => Tools::getValue('VK_STAMP'),
                'VK_DATA' => Tools::getValue('VK_DATA'),
                'VK_DATETIME' => Tools::getValue('VK_DATETIME'),
            );
        }

        $data = '';
        foreach ($array as $label) {
            $data .= $this->module->padding($label).$label;
        }

        return $data;
    }

    private function getOrderConfPage($order)
    {
        return $this->context->link->getPageLink(
            'order-confirmation',
            true,
            null,
            array(
                'id_module' => $this->module->id,
                'id_cart' => $order->id_cart,
                'key' => $order->secure_key,
            )
        );
    }

    private function sendMails($order, $type)
    {
        $customer = new Customer($order->id_customer);
      //  $template = $this->getEmailTemplate($type, $order->id_lang);

        if ($type == self::MAIL_ADMIN) {
            $emails = explode(',', $this->module->getConfig('emails'));
        } else {
            if ($type == self::MAIL_CUSTOMER) {
                $emails = array($customer->email);
            }
        }

        $contract = LhvLeaseContract::getContractByCardId($order->id_cart);
        $mail_vars = array(
            '{total}' => Tools::convertPrice($contract->down_payment + $contract->contract_fee, $order->id_currency),
            '{downpayment}' => Tools::convertPrice($contract->down_payment, $order->id_currency),
            '{contract_fee}' => Tools::convertPrice($contract->contract_fee, $order->id_currency),
            '{firstname}' => $customer->firstname,
            '{lastname}' => $customer->lastname,
            '{order_name}' => $order->reference,
        );

        foreach ($emails as $email) {
            if (!Validate::isEmail($email)) {
                continue;
            }

            Mail::send(
                $order->id_lang,
                $this->getEmailTemplate($type, $order->id_lang),
                $this->l('LHVLease contract info'),
                $mail_vars,
                trim($email),
                ($type == self::MAIL_ADMIN ? $this->context->shop->name : $customer->firstname.' '.$customer->lastname),
                $this->context->shop->name,
                Configuration::get('PS_SHOP_EMAIL'),
                null,
                null,
                $this->getEmailTemplatePath()
            );
        }
    }

    private function getEmailTemplatePath()
    {
        return $this->module->getLocalPath().'/mails/';
    }

    private function getEmailTemplate($type/*, $id_lang*/)
    {
        $template_file = '';

        if ($type == self::MAIL_CUSTOMER) {
            $template_file = 'customer';
        } else {
            if ($type == self::MAIL_ADMIN) {
                $template_file = 'admin';
            }
        }

        if ($this->contract_state == self::CONTRACT_SUCCESS) {
            $template_file .= '_success';
        } else {
            if ($this->contract_state == self::CONTRACT_PENDING) {
                $template_file .= '_pending';
            }
        }

        return $template_file;
    }

    protected function l($string, $specific = false, $class = null, $addslashes = false, $htmlentities = true)
    {
        return $this->module->l($string, 'val');
    }
}
