<?php

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

include dirname(__FILE__).'/config.inc.php';
include dirname(__FILE__).'/classes/LhvLeaseContract.php';

class LHVLease extends PaymentModule
{
    const TEST = 0;
    const LIVE = 1;

    const ORDER_STATE_SUCCESS = '_OS_LHVL_SUCCESS_';
    const ORDER_STATE_WAITING = '_OS_LHVL_WAITING_';

    const RETURN_MODULE = 0;
    const RETURN_CONF = 1;

    private $html = '';

    private $fields = array(
        'server',
        'sndid',
        'public_key',
        'private_key',
        'private_key_pass',
        'with_shipping',
        'conf_page',
        'emails',
    );
    private $lang_fields = array('payment_confirm');
    private $order_states = array(
        self::ORDER_STATE_SUCCESS => array('name' => 'LHV Lease success', 'invoice' => 1),
        self::ORDER_STATE_WAITING => array('name' => 'LHV Lease waiting', 'invoice' => 1),
    );

    private $config = null;

    //   private $is_version_16;

    public function __construct()
    {
        //GLOBAL $config;

        $this->name = 'lhvlease';
        $this->tab = 'payments_gateways';
        $this->version = '3.0.3';
        $this->author = 'LHV Finance';
        $this->controllers = array('payment', 'validation');
        $this->need_instance = 0;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('LHV Järelmaks');
        $this->description = $this->l('Tehniline tugi: kristo.meerits@lhv.ee');

        $config = null;
        include(dirname(__FILE__).'/config.inc.php');
        $this->config = $config;
    //    $this->is_version_16 = Tools::version_compare(_PS_VERSION_, 1.6, '>');
    }

    public function install()
    {
        $sql = array();
        include(dirname(__FILE__).'/sql_install.php');

        foreach ($sql as $query) {
            if (!Db::getInstance()->execute(
                sprintf(
                    $query,
                    _DB_PREFIX_,
                    $this->name,
                    _MYSQL_ENGINE_
                )
            )) {
                return false;
            }
        }

        foreach ($this->order_states as $key => $order_state) {
            if (!$this->createOrderState($key, $order_state)) {
                return false;
            }
        }

        if (!parent::install() ||
            //!$this->registerHook('displayPayment') ||
            !$this->registerHook('displayFooterProduct') ||
            !$this->registerHook('displayHeader') ||
            //!$this->registerHook('displayPaymentReturn') ||
            !$this->registerHook('paymentOptions') ||
            !$this->registerHook('paymentReturn')
        ) {
            return false;
        }

        return true;
    }

    public function getContent()
    {
        $this->postProcess();
        $this->displayForm();

        return $this->html;
    }

    private function postProcess()
    {
        if (Tools::isSubmit('submit'.$this->name)) {
            $languages = $this->context->controller->getLanguages();

            foreach ($this->fields as $field) {
                $this->updateConfig($field, Tools::getValue($field));
            }

            foreach ($this->lang_fields as $field) {
                foreach ($languages as $language) {
                    $lang_key = sprintf('%s_%s', $field, $language['id_lang']);
                    $this->updateConfig($lang_key, Tools::getValue($lang_key), true);
                }
            }

            $this->html .= $this->displayConfirmation($this->l('Settings updated'));
        }
    }

    public function hookdisplayHeader($params)
    {
        $this->context->controller->addCSS($this->_path.'views/css/lhvlease.css');
        $this->context->controller->addJS($this->_path.'views/js/lhvlease_main.js');
    }


    private function displayForm()
    {
        $helper = new HelperForm();
        $helper->languages = $this->context->controller->getLanguages();
        //  $currency = Currency::getDefaultCurrency();

        $options = array(
            array('id' => self::TEST, 'name' => $this->l('Test server')),
            array('id' => self::LIVE, 'name' => $this->l('Live server')),
        );

        $fields_form = array();
        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Settings'),
            ),
            'input' => array(
                array(
                    'type' => 'select',
                    'label' => $this->l('Ühendus'),
                    'name' => 'server',
                    'required' => true,
                    'lang' => false,
                    'options' => array(
                        'query' => $options,
                        'id' => 'id',
                        'name' => 'name',
                    ),
                ),
                array(
                    'type' => 'hidden',
                    'label' => $this->l('Admin E-mail'),
                    'name' => 'emails',
                    'hint' => $this->l('Multiple e-mail are separated by comma'),
                ),
                array(
                    'type' => 'hidden', // ($this->is_version_16 ? 'switch' : 'radio'),
                    'label' => $this->l('Include shipping'),
                    'name' => 'with_shipping',
                    'required' => false,
                    'is_bool' => true,
                    'class' => 't',
                    'values' => array(
                        array(
                            'id' => 'active_on',
                            'value' => 1,
                            'label' => $this->l('Yes'),
                        ),
                        array(
                            'id' => 'active_off',
                            'value' => 0,
                            'label' => $this->l('No'),
                        ),
                    ),
                ),
                array(
                    'type' => 'hidden',
                    'label' => $this->l('Confirmation page'),
                    'name' => 'conf_page',
                    'options' => array(
                        'query' => array(
                            array('id' => self::RETURN_MODULE, 'name' => $this->l('Module page')),
                            array('id' => self::RETURN_CONF, 'name' => $this->l('Default order confirmation page')),
                        ),
                        'id' => 'id',
                        'name' => 'name',
                    ),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('SND ID'),
                    'name' => 'sndid',
                    'required' => true,
                    'lang' => false,
                ),
                array(
                    'type' => 'textarea',
                    'rows' => 10,
                    'cols' => 100,
                    'label' => $this->l('LHV avalik võti'),
                    'name' => 'public_key',
                    'required' => true,
                    'lang' => false,
                ),
                array(
                    'type' => 'textarea',
                    'rows' => 10,
                    'cols' => 100,
                    'label' => $this->l('Sinu privaatvõti'),
                    'name' => 'private_key',
                    'required' => true,
                    'lang' => false,
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Privaatvõtme parool'),
                    'name' => 'private_key_pass',
                    'required' => false,
                    'lang' => false,
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save'),
            ),
        );

        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        $helper->default_form_language = $this->context->controller->default_form_language;
        $helper->allow_employee_form_lang = $this->context->controller->allow_employee_form_lang;

        foreach ($helper->languages as $k => $language) {
            $helper->languages[$k]['is_default'] = (int)($language['id_lang'] == $helper->default_form_language);
        }

        $helper->title = $this->displayName;
        $helper->show_toolbar = true;
        $helper->toolbar_scroll = true;
        $helper->submit_action = 'submit'.$this->name;
        $helper->toolbar_btn = array(
            'save' =>
                array(
                    'desc' => $this->l('Save'),
                    'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                        '&token='.Tools::getAdminTokenLite('AdminModules'),
                ),
            'back' => array(
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list'),
            ),
        );

        $helper->fields_value = array();
        foreach ($this->fields as $input) {
            $helper->fields_value[$input] = $this->getConfig($input);
        }

        foreach ($this->lang_fields as $field) {
            $helper->fields_value[$field] = array();
            foreach ($helper->languages as $language) {
                $helper->fields_value[$field][$language['id_lang']] = $this->getConfig(
                    sprintf(
                        '%s_%s',
                        $field,
                        $language['id_lang']
                    )
                );
            }
        }

        $this->html .= $helper->generateForm($fields_form);
    }

    private function prefixed($string)
    {
        return Tools::strtoupper(sprintf('%s_%s', $this->name, $string));
    }

    public function getConfig($key, $default = false)
    {
        return Configuration::get($this->prefixed($key), $default);
    }

    public function updateConfig($key, $value, $allow_html = false)
    {
        return Configuration::updateValue($this->prefixed($key), $value, $allow_html);
    }

    private function createOrderState($key, $state)
    {
        $orderStateExist = false;
        //   $logoCopied = false;
        $orderStateId = Configuration::get($key);

        if ($orderStateId) {
            $orderState = new OrderState($orderStateId);
            if ($orderState->id && !$orderState->deleted) {
                $orderStateExist = true;
            }
        } else {
            $query = 'SELECT os.`id_order_state` '.
                'FROM `%1$sorder_state_lang` osl '.
                'LEFT JOIN `%1$sorder_state` os '.
                'ON osl.`id_order_state`=os.`id_order_state` '.
                'WHERE osl.`name`="%2$s" AND os.`deleted`=0';
            $orderStateId = Db::getInstance()->getValue(sprintf($query, _DB_PREFIX_, $state['name']));
            if ($orderStateId) {
                Configuration::updateValue($key, $orderStateId);
                $orderStateExist = true;
            }
        }

        if (!$orderStateExist) {
            $languages = Language::getLanguages(false);
            $orderState = new OrderState();
            foreach ($languages as $lang) {
                $orderState->name[$lang['id_lang']] = $state['name'];
            }
            $orderState->send_email = 0;
            $orderState->invoice = $state['invoice'];
            $orderState->color = "lightblue";
            $orderState->unremovable = 0;
            $orderState->logable = 0;
            $orderState->delivery = 0;
            $orderState->hidden = 0;

            if ($orderState->add()) {
                Configuration::updateValue($key, $orderState->id);
                $orderStateExist = true;
            }
        }


        //$file = $this->getLocalPath().'/logo.gif';
        //$newfile = _PS_IMG_DIR_.'os/'.$orderState->id.'.gif';

        return $orderStateExist;
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        $newOption = new PaymentOption();
        $newOption->setModuleName($this->name)
            ->setCallToActionText($this->l('LHV järelmaks'))
            ->setAction($this->context->link->getModuleLink($this->name, 'confirmation', array(), true))
            ->setAdditionalInformation($this->context->smarty->fetch(
                'module:lhvlease/views/templates/hook/payment.tpl'
            ));

        $payment_options = [
            $newOption,
        ];

        return $payment_options;
    }

    /*public function hookDisplayPayment($params)
    {
        if (!$this->active)
            return;

        $this->smarty->assign(array(
            'lang_iso_code' => $this->context->language->iso_code,
            'this_path' => $this->_path,
            'href' => $this->context->link->getModuleLink($this->name, 'confirmation')
        ));
        return $this->display(__FILE__, 'payment.tpl');
    }*/

    // public function hookdisplayFooterProduct($params)
    // {
    //     if (!$this->active) {
    //         return;
    //     }

    //     $this->smarty->assign(array(
    //         'this_path' => $this->_path,
    //         'href' => $this->context->link->getModuleLink($this->name, 'confirmation'),
    //     ));
    //     $this->context->controller->addJS(_THEME_JS_DIR_.'payment-lease-calculator.js');

    //     return $this->context->smarty->fetch('module:lhvlease/views/templates/hook/paymentFooter.tpl');
    // }

    public function hookPaymentReturn($params)
    {
        if ($this->hasOrderState($params['order'], LHVLease::ORDER_STATE_SUCCESS)) {
            $template = 'success.tpl';
        } else {
            if ($this->hasOrderState($params['order'], LHVLease::ORDER_STATE_WAITING)) {
                $template = 'pending.tpl';
            } else {
                $template = 'nothing.tpl';
            }
        }

        return $this->context->smarty->fetch('module:lhvlease/views/templates/front/'.$template);
    }

    /*public function hookDisplayPaymentReturn($params)
    {
        if ($this->hasOrderState($params['objOrder'], LHVLease::ORDER_STATE_SUCCESS))
        {
            $template = 'success.tpl';
        }
        else if ($this->hasOrderState($params['objOrder'], LHVLease::ORDER_STATE_WAITING))
        {
            $template = 'pending.tpl';
        }
        else
        {
            $template = 'nothing.tpl';
        }

        return $this->display(__FILE__, $template);
    }*/

    public function getConfigFromFile($server = null, $key = null)
    {
        if (!is_null($server) && !empty($key)) {
            return $this->config[$server][$key];
        } elseif (!is_null($server)) {
            return $this->config[$server];
        } else {
            return $this->config;
        }
    }

    public function padding($string)
    {
        return str_pad(mb_strlen($string, 'UTF-8'), 3, '0', STR_PAD_LEFT);
    }

    public function hasOrderState($order, $order_state)
    {
        $id_order_state = Configuration::get($order_state);
        $result = false;
        if (Validate::isUnsignedInt($id_order_state)) {
            $order_state = $order->getHistory(
                $this->context->language->id,
                $id_order_state
            );

            $result = !(empty($order_state));
        }

        return $result;
    }
}
