/**
 * Default copyright laws apply.
 *
 * Veebipoed retains all rights to this source code and nobody else may reproduce, distribute
 * or create derivative works from this work.
 *
 * Permitted:
 * - Commercial Use
 * This software and derivatives may be used for commercial purposes.
 * - Private Use
 * You may use and modify the software without distributing it.
 *
 * Forbidden:
 * - Distribution
 * You may not distribute this software.
 * - Modification
 * This software may not be modified.
 * - Sublicensing
 * You may not grant a sublicense to modify and distribute this software to third parties not included in the license.
 *
 *  @author    Veebipoed.ee
 *  @copyright 2020 Veebipoed.ee
 *  @license   Veebipoed.ee
 */
if (!LHV) {
    var LHV = {}
}
LHV.l10n = {};
var _ = function (e) {
    if (typeof LHV === "undefined" || typeof LHV.l10n === "undefined" || typeof LHV.l10n[e] === "undefined") {
        return "[MISSING L10N]"
    }
    var t = LHV.l10n[e];
    return t
};
LHV.Widgets = function () {
    return {
        roundNumber: function (e, t) {
            var n = parseFloat(Math.round(e * 100) / 100, 10).toFixed(t);
            return n
        }
    }
}();
LHV.HirePurchase = function () {
    var hpApp = this;
    var $container, $priceField, $advanceField, $periodField, $intrestRateField, $loanAmountField, $advanceAmountField,
        $aprField;
    var isNumeric = function (value) {
        value = value + '';
        value.replace(/,/g, '.');
        value.replace(/[^0-9.]/g, '');
        return (value != '' && !isNaN(value))
    };
    hpApp.init = function ($cont) {
        $container = $cont;
        $priceField = $('[name="price"]', $container);
        $advanceField = $('[name="advance"]', $container);
        $periodField = $('[name="period"]', $container);
        $intrestRateField = $('[name="intrest_rate"]', $container);
        $loanAmountField = $('[name="loan_amount"]', $container);
        $advanceAmountField = $('[name="advance_amount"]', $container);
        $aprField = $('[name="apr"]', $container);
        var $typingTimeout;
        $intrestRateField.on('keyup change', function (e) {
            if (e.altKey || e.metaKey || e.ctrlKey || e.shiftKey || (e.keyCode >= 37 && e.keyCode <= 40)) {
                return
            }
            var field = this;
            clearTimeout($typingTimeout);
            $typingTimeout = setTimeout(function () {
                hpApp.clearError();
                field.value = field.value.replace(/,/g, '.');
                if (isNumeric(field.value)) {
                    if (field.value >= 15.9 && field.value <= 29.9) {
                        hpApp.showResults()
                    } else {
                        hpApp.showError(_('invalidinterestraterange'))
                    }
                } else if (field.value === '') {
                    hpApp.clearError()
                } else {
                    hpApp.showError(_('invalidinterestrate'))
                }
            }, 250)
        });
        $periodField.on('change', function () {
            hpApp.clearError();
            hpApp.showResults()
        });
        $advanceField.on('change', function () {
            hpApp.clearError();
            hpApp.showResults()
        });
        $priceField.on('keyup change', function (e) {
            if (e.altKey || e.metaKey || e.ctrlKey || e.shiftKey || (e.keyCode >= 37 && e.keyCode <= 40)) {
                return
            }
            var field = this;
            clearTimeout($typingTimeout);
            $typingTimeout = setTimeout(function () {
                hpApp.clearError();
                field.value = field.value.replace(/,/g, '.');
                if (isNumeric(field.value)) {
                    if (field.value >= 50 && field.value <= 10000) {
                        hpApp.showResults()
                    } else {
                        hpApp.showError(_('invalidpricerange'))
                    }
                } else if (field.value === '') {
                    hpApp.clearError()
                } else {
                    hpApp.showError(_('invalidprice'))
                }
            }, 250)
        })
    };
    hpApp.hasErrors = function () {
        var messages = '';
        $priceField.each(function () {
            var field = this;
            if (isNumeric(field.value)) {
                if (field.value < 50 || field.value > 10000) {
                    messages += _('invalidpricerange')
                }
            }
        });
        $intrestRateField.each(function () {
            var field = this;
            if (isNumeric(field.value)) {
                if (field.value < 15.9 || field.value > 29.9) {
                    messages += _('invalidinterestraterange')
                }
            }
        });
        hpApp.showError(messages);
        if (messages.length == 0) {
            return false
        } else {
            return true
        }
    };
    hpApp.showResults = function () {
        if ($priceField.val() !== '' && $intrestRateField.val() !== '') {
            var advanceAmount = parseFloat($priceField.val(), 10) * parseFloat($advanceField.val(), 10) / 100;
            var payment = hpApp.calculatePayment(parseFloat($priceField.val(), 10), parseFloat(advanceAmount, 10), parseInt($periodField.val(), 10), parseFloat($intrestRateField.val(), 10), 0);
            if (!hpApp.hasErrors()) {
                if (payment >= 10) {
                    $('.results-separator, .calculation-results', $container).removeClass('hidden');
                    $loanAmountField.val(payment);
                    $advanceAmountField.val(advanceAmount);
                    $('.loan-amount-text', $container).text(LHV.Widgets.roundNumber(payment, 2));
                    $('.advance-amount-text', $container).text(LHV.Widgets.roundNumber(advanceAmount, 2));
                    $('.apr-amount-text', $container).text(LHV.Widgets.roundNumber(100, 2))
                } else {
                    hpApp.showError(_('paymentoolow'))
                }
            }
        } else if ($priceField.val() === '' && $intrestRateField.val() === '') {
            hpApp.clearError()
        } else {
            hpApp.showError(_('missingfields'))
        }
    };
    hpApp.showError = function (message) {
        $('.results-separator, .error', $container).removeClass('hidden');
        $('.no-error', $container).addClass('hidden');
        $('.error', $container).text(message)
    };
    hpApp.clearError = function () {
        $('.results-separator, .calculation-results, .error', $container).addClass('hidden');
        $('.no-error', $container).removeClass('hidden')
    };
    hpApp.calculatePayment = function (price, advance, period, interestRate, residualValue) {
        var payment;
        if (interestRate > 0) {
            payment = (((interestRate / 100 / 12) * ((price - advance) - ((price * residualValue / 100) / (Math.pow((interestRate / 100 / 12) + 1, period)))) / (1 - (1 / Math.pow((interestRate / 100 / 12) + 1, period)))))
        } else {
            payment = (price - advance - (price * residualValue / 100)) / period
        }
        return payment
    }
};

LHV.l10n.invalidprice = "Eseme hind peab olema number.";
LHV.l10n.invalidpricerange = "Eseme hind võib olla vahemikus 50 – 10 000 eurot.";
LHV.l10n.invalidinterestrate = "Intress peab olema number.";
LHV.l10n.invalidinterestraterange = "Intress peab peab jääma vahemikku 15,9%–29,9%.";
LHV.l10n.paymentoolow = "Kuumakse ei saa olla vähem kui 10 eurot, muuda sissemakset või tagasimakse perioodi.";
LHV.l10n.missingfields = "Täida kõik väljad.";


$('.hire-purchase.calculator-widget').each(function () {
    var app = new LHV.HirePurchase();
    app.init($(this));
});

$(document).ready(function () {
    var curTotal = $('.cart-total > .value').text();
    curTotal = curTotal.replace(/\,/g, '.').replace(/\€/g, '');
    $('#lhvPriceField').val(curTotal);
});
